/*
 * This file is part of SCHUR.
 *
 * SCHUR - an interactive program for calculating properties of Lie
 * groups and symmetric functions.
 * Copyright (C) 2006  Franck BUTELLE, Frédéric Toumazet
 * 
 * SCHUR is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * SCHUR is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SCHUR; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "sets_mgmt.h"

setptr
Currset (int n, setptr sp)
{
  static setword Space[SETSPACE];
  static setptr Top = Space;

switch (n)
  {
  case 0:
    Top = Space;
    return (0);
  case 1:
    if (&Space[SETSPACE] - Top <= setbits)
      {
	(void) fprintf (stderr, "Set-space exhausted\n");
	exit (1);
      }
    *Top = 0;
    return (Top);
  case 2:
    if (Top <= &sp[*sp])
      Top = &sp[*sp + 1];
    return (sp);
  }
return (0);
}

setptr
Union (register setptr p1, register setptr p2)
{
  register int ig, jg, k;
  register setptr sp = Newset (), p3 = sp;

  jg = *p1;
  *p3 = jg;
  if (jg > *p2)
    jg = *p2;
  else
    *p3 = *p2;
  k = *p1 - *p2;
  p1++, p2++, p3++;
  for (ig = 0; ig < jg; ig++)
    *p3++ = (*p1++ | *p2++);
  while (k > 0)
    {
      *p3++ = *p1++;
      k--;
    }
  while (k < 0)
    {
      *p3++ = *p2++;
      k++;
    }
  return (Saveset (sp));
}

bool
Member (register unsigned int m, register setptr sp)
{
    register unsigned int ig = m / (setbits + 1) + 1;

      if ((ig <= *sp) && (sp[ig] & (1 << (m % (setbits + 1)))))
	    return (true);
        return (false);
}
